#ifndef _CSimpleFilter_h_
#define _CSimpleFilter_h_

#include <streams.h>

#define SAFE_DELETE(p) { if(p!=NULL) { delete (p); (p)=NULL; } }
#define SAFE_RELEASE(p) { if(p!=NULL) { (p)->Release(); (p)=NULL; } }

#define ONE_SECOND 10000000
#define INFINITE_SECOND 0x7fffffffffffffff
// --------------------------------------------------------------------------------------------------------------------
// fR[hݒC^[tFCX
// {51296C64-6A98-462c-8114-2259C056C96C}
static const GUID IID_IDecodeInterface =
{ 0x51296c64, 0x6a98, 0x462c, { 0x81, 0x14, 0x22, 0x59, 0xc0, 0x56, 0xc9, 0x6c } };

__interface
IDecodeInterface : public IUnknown
{
public:
   HRESULT GetOutFormat(void *);
   HRESULT SetOutFormat(void *);
};
// --------------------------------------------------------------------------------------------------------------------
class CSimpleTransform : public CTransformFilter
{
public:
   DECLARE_IUNKNOWN

   // RXgN^ƃfXgN^
   CSimpleTransform(__in_opt LPCTSTR, __inout_opt LPUNKNOWN, REFCLSID, HRESULT *);
   ~CSimpleTransform();

   // z\bh
   virtual HRESULT OnConnectInPin(const CMediaType *) PURE;
   virtual HRESULT OnConnectOutPin(const CMediaType *, int, CMediaType *, int *) PURE;

   // I[o[Ch\ȃ\bh
   virtual HRESULT OnReceive(IMediaSample *);
   virtual HRESULT OnTransform(IMediaSample *, IMediaSample *);
   virtual STDMETHODIMP OnQueryInterface(REFIID riid, void ** ppv);
   virtual HRESULT OnStart(void);
   virtual HRESULT OnStop(bool);
   virtual HRESULT OnSeek(void);

   // CTransformFilteȑz\bh
   HRESULT CheckInputType(const CMediaType *);
   HRESULT CheckTransform(const CMediaType *, const CMediaType *);
   HRESULT DecideBufferSize(IMemAllocator *, ALLOCATOR_PROPERTIES *);
   HRESULT GetMediaType(int, CMediaType *);

   // CTransformFilter̃I[o[Ch
   HRESULT Receive(IMediaSample *);
   HRESULT BreakConnect(PIN_DIRECTION dir);
   STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv);
   virtual HRESULT StartStreaming();
   virtual HRESULT StopStreaming();
   virtual HRESULT EndOfStream(void);
   HRESULT BeginFlush();

private:

   // [Uϐ
   CMediaType m_mtOut;
   int m_nOutBufferSize;
   CCritSec m_Lock;
};
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
class CSimpleInPlace : public CTransInPlaceFilter
{
public:
   DECLARE_IUNKNOWN

   // RXgN^ƃfXgN^
   CSimpleInPlace(__in_opt LPCTSTR, __inout_opt LPUNKNOWN, REFCLSID, HRESULT *, bool);
   ~CSimpleInPlace();

   // z\bh
   virtual HRESULT OnConnectInPin(const CMediaType *) PURE;
   virtual HRESULT OnTransform(IMediaSample *) PURE;

   // I[o[Ch\ȃ\bh
   virtual STDMETHODIMP OnQueryInterface(REFIID riid, void ** ppv);
   virtual HRESULT OnStart(void);
   virtual HRESULT OnStop(bool);
   virtual HRESULT OnSeek(void);

   // CTransInPlaceFilteȑz\bh
   HRESULT CheckInputType(const CMediaType *pmtIn)
   {
      CAutoLock lck(&m_Lock);
      return OnConnectInPin(pmtIn);
   }

   HRESULT Transform(IMediaSample *pInSample)
   {
      CAutoLock lck(&m_Lock);
      return OnTransform(pInSample);
   }

   // CTransInPlaceFilter̃I[o[Ch
   STDMETHODIMP NonDelegatingQueryInterface(REFIID, void **);

   HRESULT StartStreaming()
   {
      return OnStart();
   }

   HRESULT StopStreaming()
   {
      CAutoLock lck(&m_Lock);
      return OnStop(false);
   }

   HRESULT EndOfStream(void)
   {
      CAutoLock lck(&m_Lock);
      return OnStop(true);
   }

private:

   // [Uϐ
   CCritSec m_Lock;
   CPosPassThru *m_pPosition;
};
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
class CSimpleWriterInputPin : public CRenderedInputPin
{
   friend class CSimpleWriter;

public:

   // RXgN^ƃfXgN^
   CSimpleWriterInputPin(CSimpleWriter *, LPUNKNOWN,CCritSec *, CCritSec *, HRESULT *);
   ~CSimpleWriterInputPin();

   // CRenderedInputPin(CBasePin)̏z\bh
   virtual HRESULT CheckMediaType(const CMediaType *);

   // CRenderedInputPiñI[o[Ch
   STDMETHODIMP Receive(IMediaSample *);
   STDMETHODIMP EndOfStream(void);
   STDMETHODIMP ReceiveCanBlock() { return S_FALSE;}
   STDMETHODIMP BeginFlush();
   HRESULT BreakConnect();

   STDMETHODIMP NewSegment(REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate)
   {
      return S_OK;
   }

private:

   // [Uϐ
   CSimpleWriter * const m_pFilter;
   CCritSec *m_pLock;
};
// -----------------------------------------------------------------------------------------------------------------------------------
class CSimpleWriter : public CBaseFilter , public  IFileSinkFilter
{
   friend class CSimpleWriterInputPin;

protected:
   CSimpleWriterInputPin *m_pInput;
   WCHAR m_awInputFileName[MAX_PATH];

   HANDLE m_hFile;
   CCritSec m_Lock;

public:
   DECLARE_IUNKNOWN

   // RXgN^ƃfXgN^
   CSimpleWriter::CSimpleWriter(__in_opt LPCTSTR, __inout_opt LPUNKNOWN, REFCLSID, HRESULT *);
   ~CSimpleWriter();

   // z\bh
   virtual HRESULT OnConnectInPin(const CMediaType *) PURE;

   // I[o[Ch\ȃ\bh
   virtual HRESULT OnReceive(HANDLE, IMediaSample *);
   virtual STDMETHODIMP OnQueryInterface(REFIID riid, void ** ppv);
   virtual HRESULT OnStart(HANDLE);
   virtual HRESULT OnStop(HANDLE, bool);
   virtual HRESULT OnSeek(HANDLE);

   // CBaseFilteȑz\bh
   virtual CBasePin *GetPin(int n);
   virtual int GetPinCount();

   // CBaseFilter̃I[o[Ch
   STDMETHODIMP NonDelegatingQueryInterface(REFIID, void **);
   STDMETHODIMP Stop();

   // IFileSinkFilteȑz\bh
   STDMETHODIMP SetFileName(LPCOLESTR pszFileName, const AM_MEDIA_TYPE *pmt);
   STDMETHODIMP GetCurFile(LPOLESTR *ppszFileName, AM_MEDIA_TYPE *pmt);

private:

   // [Uϐ
   CCritSec m_ReceiveLock;
   CPosPassThru *m_pPosition;

   bool m_bFirstReceive;
};
// -----------------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------------
class CSimplePushSourceOutputPin : public CSourceStream
{
   friend class CSimplePushSource;

public:

   // RXgN^ƃfXgN^
   CSimplePushSourceOutputPin(CSimplePushSource *, CCritSec *, HRESULT *);
   ~CSimplePushSourceOutputPin();

   // CSourceStream̏z\bh
   HRESULT DecideBufferSize(IMemAllocator *, ALLOCATOR_PROPERTIES *);
   HRESULT FillBuffer(IMediaSample *);
 
   // CSourceStream̃I[o[Ch
   HRESULT GetMediaType(CMediaType *);
   HRESULT CheckMediaType(const CMediaType *);
   HRESULT OnThreadCreate(void);
   HRESULT OnThreadDestroy(void);

private:  

   // [Uϐ
   int m_nOutBufferSize;
   CCritSec *m_pMyLock;
};
// -----------------------------------------------------------------------------------------------------------------------------------
class CSimplePushSource : public CSource, public IMediaSeeking
{
   friend class CSimplePushSourceOutputPin;

public:
   DECLARE_IUNKNOWN

   // RXgN^ƃfXgN^
   CSimplePushSource(__in_opt LPCTSTR, __inout_opt LPUNKNOWN, REFCLSID, HRESULT *);
   ~CSimplePushSource();

   // z\bh
   virtual HRESULT OnConnectOutPin(CMediaType *, int *, REFERENCE_TIME *) PURE;
   virtual HRESULT FillBuffer(IMediaSample *) PURE;

   // I[o[Ch\ȃ\bh
   virtual STDMETHODIMP OnQueryInterface(REFIID riid, void ** ppv);
   virtual HRESULT OnStart(void);
   virtual HRESULT OnStop(void);
   virtual HRESULT OnSeek(LONGLONG, LONGLONG);

   // CUnknowñI[o[Ch
   STDMETHODIMP NonDelegatingQueryInterface(REFIID, void **);

   // IMediaSeeking
   HRESULT STDMETHODCALLTYPE GetDuration(LONGLONG *);
   HRESULT STDMETHODCALLTYPE GetCurrentPosition(LONGLONG *);
   HRESULT STDMETHODCALLTYPE GetCapabilities(DWORD *);
   HRESULT STDMETHODCALLTYPE CheckCapabilities(DWORD *);
   HRESULT STDMETHODCALLTYPE IsFormatSupported(const GUID *);
   HRESULT STDMETHODCALLTYPE QueryPreferredFormat(GUID *);
   HRESULT STDMETHODCALLTYPE GetTimeFormat(GUID *);
   HRESULT STDMETHODCALLTYPE IsUsingTimeFormat(const GUID *);
   HRESULT STDMETHODCALLTYPE SetTimeFormat(const GUID *);
   HRESULT STDMETHODCALLTYPE GetStopPosition(LONGLONG *);
   HRESULT STDMETHODCALLTYPE ConvertTimeFormat(LONGLONG *, const GUID *, LONGLONG, const GUID *);
   HRESULT STDMETHODCALLTYPE SetPositions(LONGLONG *, DWORD, LONGLONG *, DWORD);
   HRESULT STDMETHODCALLTYPE GetPositions(LONGLONG *, LONGLONG *);
   HRESULT STDMETHODCALLTYPE GetAvailable(LONGLONG *, LONGLONG *);
   HRESULT STDMETHODCALLTYPE SetRate(double);
   HRESULT STDMETHODCALLTYPE GetRate(double *);
   HRESULT STDMETHODCALLTYPE GetPreroll(LONGLONG *);

private:

   // [Uϐ
   CCritSec m_MyLock;
   WCHAR m_awInputFileName[MAX_PATH];
   CSimplePushSourceOutputPin *m_pOutPin;
   REFERENCE_TIME m_rtStopTime, m_rtTotalTime;
   REFERENCE_TIME m_rtStart, m_rtEnd;
};
// -----------------------------------------------------------------------------------------------------------------------------------

#endif _CSimpleFilter_h_
